from _typeshed import Incomplete
from typing import Any
from typing_extensions import Literal, TypeAlias

from .Image import Image

# imported from either of {PyQt6,PySide6,PyQt5,PySide2}.QtGui
# These are way too complex, with 4 different possible sources (2 deprecated)
# And we don't want to force the user to install PyQt or Pyside when they may not even use it.
_QImage: TypeAlias = Any
_QPixmap: TypeAlias = Any

qt_versions: Incomplete
qt_is_installed: bool
qt_version: Incomplete

def rgb(r: int, g: int, b: int, a: int = 255) -> int: ...
def fromqimage(im: ImageQt | _QImage) -> Image: ...
def fromqpixmap(im: ImageQt | _QImage) -> Image: ...
def align8to32(bytes: bytes, width: int, mode: Literal["1", "L", "P"]) -> bytes: ...

class ImageQt(_QImage):
    def __init__(self, im: Image) -> None: ...

def toqimage(im: Image) -> ImageQt: ...
def toqpixmap(im: Image) -> _QPixmap: ...



from _typeshed import Incomplete, Unused
from enum import IntEnum
from typing import ClassVar
from typing_extensions import Literal

from ._binary import o8 as o8
from .ImageFile import ImageFile

is_cid: Incomplete
MAX_TEXT_CHUNK: Incomplete
MAX_TEXT_MEMORY: Incomplete

class Disposal(IntEnum):
    OP_NONE: int
    OP_BACKGROUND: int
    OP_PREVIOUS: int

class Blend(IntEnum):
    OP_SOURCE: int
    OP_OVER: int

class ChunkStream:
    fp: Incomplete
    queue: Incomplete
    def __init__(self, fp) -> None: ...
    def read(self): ...
    def __enter__(self): ...
    def __exit__(self, *args: Unused) -> None: ...
    def close(self) -> None: ...
    def push(self, cid, pos, length) -> None: ...
    def call(self, cid, pos, length): ...
    def crc(self, cid, data) -> None: ...
    def crc_skip(self, cid, data) -> None: ...
    def verify(self, endchunk: bytes = b"IEND"): ...

class iTXt(str):
    lang: Incomplete
    tkey: Incomplete
    @staticmethod
    def __new__(cls, text, lang: Incomplete | None = None, tkey: Incomplete | None = None): ...

class PngInfo:
    chunks: Incomplete
    def __init__(self) -> None: ...
    def add(self, cid, data, after_idat: bool = False) -> None: ...
    def add_itxt(self, key, value, lang: str = "", tkey: str = "", zip: bool = False) -> None: ...
    def add_text(self, key, value, zip: bool = False): ...

class PngStream(ChunkStream):
    im_info: Incomplete
    im_text: Incomplete
    im_size: Incomplete
    im_mode: Incomplete
    im_tile: Incomplete
    im_palette: Incomplete
    im_custom_mimetype: Incomplete
    im_n_frames: Incomplete
    rewind_state: Incomplete
    text_memory: int
    def __init__(self, fp) -> None: ...
    def check_text_memory(self, chunklen) -> None: ...
    def save_rewind(self) -> None: ...
    def rewind(self) -> None: ...
    def chunk_iCCP(self, pos, length): ...
    def chunk_IHDR(self, pos, length): ...
    im_idat: Incomplete
    def chunk_IDAT(self, pos, length) -> None: ...
    def chunk_IEND(self, pos, length) -> None: ...
    def chunk_PLTE(self, pos, length): ...
    def chunk_tRNS(self, pos, length): ...
    def chunk_gAMA(self, pos, length): ...
    def chunk_cHRM(self, pos, length): ...
    def chunk_sRGB(self, pos, length): ...
    def chunk_pHYs(self, pos, length): ...
    def chunk_tEXt(self, pos, length): ...
    def chunk_zTXt(self, pos, length): ...
    def chunk_iTXt(self, pos, length): ...
    def chunk_eXIf(self, pos, length): ...
    def chunk_acTL(self, pos, length): ...
    def chunk_fcTL(self, pos, length): ...
    def chunk_fdAT(self, pos, length): ...

class PngImageFile(ImageFile):
    format: ClassVar[Literal["PNG"]]
    format_description: ClassVar[str]
    @property
    def text(self): ...
    fp: Incomplete
    def verify(self) -> None: ...
    def seek(self, frame) -> None: ...
    def tell(self): ...
    decoderconfig: Incomplete
    def load_prepare(self) -> None: ...
    def load_read(self, read_bytes): ...
    png: Incomplete
    im: Incomplete
    def load_end(self) -> None: ...
    def getexif(self): ...
    def getxmp(self): ...

def putchunk(fp, cid, *data) -> None: ...

class _idat:
    fp: Incomplete
    chunk: Incomplete
    def __init__(self, fp, chunk) -> None: ...
    def write(self, data) -> None: ...

class _fdat:
    fp: Incomplete
    chunk: Incomplete
    seq_num: Incomplete
    def __init__(self, fp, chunk, seq_num) -> None: ...
    def write(self, data) -> None: ...

def getchunks(im, **params): ...



from _typeshed import Incomplete
from typing import ClassVar
from typing_extensions import Literal

from .ImageFile import ImageFile, PyDecoder

MODES: Incomplete

class SgiImageFile(ImageFile):
    format: ClassVar[Literal["SGI"]]
    format_description: ClassVar[str]

class SGI16Decoder(PyDecoder):
    def decode(self, buffer): ...



from _typeshed import Incomplete
from typing import ClassVar
from typing_extensions import Literal

from .ImageFile import ImageFile

field: Incomplete

class ImtImageFile(ImageFile):
    format: ClassVar[Literal["IMT"]]
    format_description: ClassVar[str]



from _typeshed import Incomplete
from typing import ClassVar
from typing_extensions import Literal

from .ImageFile import ImageFile

MODES: Incomplete

class PsdImageFile(ImageFile):
    format: ClassVar[Literal["PSD"]]
    format_description: ClassVar[str]
    mode: Incomplete
    tile: Incomplete
    frame: Incomplete
    fp: Incomplete
    def seek(self, layer): ...
    def tell(self): ...
    im: Incomplete
    def load_prepare(self) -> None: ...



from .Image import Image

def constant(image: Image, value: int) -> Image: ...
def duplicate(image: Image) -> Image: ...
def invert(image: Image) -> Image: ...
def lighter(image1: Image, image2: Image) -> Image: ...
def darker(image1: Image, image2: Image) -> Image: ...
def difference(image1: Image, image2: Image) -> Image: ...
def multiply(image1: Image, image2: Image) -> Image: ...
def screen(image1: Image, image2: Image) -> Image: ...
def soft_light(image1: Image, image2: Image) -> Image: ...
def hard_light(image1: Image, image2: Image) -> Image: ...
def overlay(image1: Image, image2: Image) -> Image: ...
def add(image1: Image, image2: Image, scale: float = 1.0, offset: int = 0) -> Image: ...
def subtract(image1: Image, image2: Image, scale: float = 1.0, offset: int = 0) -> Image: ...
def add_modulo(image1: Image, image2: Image) -> Image: ...
def subtract_modulo(image1: Image, image2: Image) -> Image: ...
def logical_and(image1: Image, image2: Image) -> Image: ...
def logical_or(image1: Image, image2: Image) -> Image: ...
def logical_xor(image1: Image, image2: Image) -> Image: ...
def blend(image1: Image, image2: Image, alpha: float) -> Image: ...
def composite(image1: Image, image2: Image, mask: Image) -> Image: ...
def offset(image: Image, xoffset: int, yoffset: int | None = None) -> Image: ...



from _typeshed import Incomplete
from typing import ClassVar
from typing_extensions import Literal

from .ImageFile import ImageFile, PyDecoder

b_whitespace: bytes
MODES: Incomplete

class PpmImageFile(ImageFile):
    format: ClassVar[Literal["PPM"]]
    format_description: ClassVar[str]

class PpmPlainDecoder(PyDecoder):
    def decode(self, buffer): ...

class PpmDecoder(PyDecoder):
    def decode(self, buffer): ...



from _typeshed import Incomplete

WIDTH: int

def puti16(fp, values) -> None: ...

class FontFile:
    bitmap: Incomplete
    info: Incomplete
    glyph: Incomplete
    def __init__(self) -> None: ...
    def __getitem__(self, ix): ...
    ysize: Incomplete
    metrics: Incomplete
    def compile(self): ...
    def save(self, filename) -> None: ...



from _typeshed import Incomplete, Unused
from typing import Any
from typing_extensions import Literal

def register(viewer, order: int = 1) -> None: ...
def show(image, title: Incomplete | None = None, **options): ...

class Viewer:
    def show(self, image, **options): ...
    format: Incomplete
    options: Incomplete
    def get_format(self, image): ...
    # Abstract method
    def get_command(self, file: str, **options: Any) -> str: ...
    def save_image(self, image): ...
    def show_image(self, image, **options): ...
    def show_file(self, path: str, **options: Any) -> Literal[1]: ...

class WindowsViewer(Viewer):
    format: str
    options: Incomplete
    def get_command(self, file: str, **options: Unused) -> str: ...

class MacViewer(Viewer):
    format: str
    options: Incomplete
    def get_command(self, file: str, **options: Unused) -> str: ...
    def show_file(self, path: str, **options: Unused) -> Literal[1]: ...

class UnixViewer(Viewer):
    format: str
    options: Incomplete
    # Unix viewer expects the following method to be implemented on all
    # subclasses, but it isn't defined on the base class.
    # def get_command_ex(self, file: str, **options: Any) -> tuple[str, str]: ...
    def get_command(self, file: str, **options: Any) -> str: ...

class XDGViewer(UnixViewer):
    def get_command_ex(self, file: str, **options: Unused) -> tuple[str, str]: ...
    def show_file(self, path: str, **options: Unused) -> Literal[1]: ...

class DisplayViewer(UnixViewer):
    def get_command_ex(self, file: str, title: str | None = None, **options: Unused) -> tuple[str, str]: ...
    def show_file(self, path: str, *, title: str | None = None, **options: Unused) -> Literal[1]: ...

class GmDisplayViewer(UnixViewer):
    def get_command_ex(self, file: str, **options: Unused) -> tuple[str, str]: ...
    def show_file(self, path: str, **options: Unused) -> Literal[1]: ...

class EogViewer(UnixViewer):
    def get_command_ex(self, file: str, **options: Unused) -> tuple[str, str]: ...
    def show_file(self, path: str, **options: Unused) -> Literal[1]: ...

class XVViewer(UnixViewer):
    def get_command_ex(self, file, title: str | None = None, **options: Unused) -> tuple[str, str]: ...
    def show_file(self, path: str, *, title: str | None = None, **options: Unused) -> Literal[1]: ...

class IPythonViewer(Viewer):
    def show_image(self, image, **options: Unused) -> Literal[1]: ...



from _typeshed import Incomplete

class Stat:
    h: Incomplete
    bands: Incomplete
    def __init__(self, image_or_list, mask: Incomplete | None = None) -> None: ...
    def __getattr__(self, id: str): ...

Global = Stat



import _tkinter
import tkinter
from _typeshed import ReadableBuffer, StrOrBytesPath, SupportsRead
from typing import Any

from PIL.Image import Image, _Mode, _Size

class PhotoImage(tkinter._PhotoImageLike):
    tk: _tkinter.TkappType
    def __init__(
        self,
        image: Image | _Mode | None = None,
        size: _Size | None = None,
        *,
        file: StrOrBytesPath | SupportsRead[bytes] = ...,
        data: ReadableBuffer = ...,
        # These are forwarded to tkinter.PhotoImage.__init__():
        name: str | None = None,
        cnf: dict[str, Any] = ...,
        format: str = ...,
        gamma: float = ...,
        height: int = ...,
        palette: int | str = ...,
        width: int = ...,
    ) -> None: ...
    def __del__(self) -> None: ...
    def width(self) -> int: ...
    def height(self) -> int: ...
    def paste(self, im: Image) -> None: ...

class BitmapImage(tkinter._BitmapImageLike):
    def __init__(
        self,
        image: Image | None = None,
        *,
        file: StrOrBytesPath | SupportsRead[bytes] = ...,
        data: ReadableBuffer = ...,
        # These are forwarded to tkinter.Bitmap.__init__():
        name: str | None = None,
        cnf: dict[str, Any] = ...,
        master: tkinter.Misc | _tkinter.TkappType | None = None,
        background: str = ...,
        foreground: str = ...,
        maskdata: str = ...,
        maskfile: StrOrBytesPath = ...,
    ) -> None: ...
    def __del__(self) -> None: ...
    def width(self) -> int: ...
    def height(self) -> int: ...

def getimage(photo: tkinter.PhotoImage) -> Image: ...



from typing import ClassVar
from typing_extensions import Final, Literal

from .ImageFile import ImageFile

DDS_MAGIC: Final = 0x20534444

DDSD_CAPS: Final = 0x1
DDSD_HEIGHT: Final = 0x2
DDSD_WIDTH: Final = 0x4
DDSD_PITCH: Final = 0x8
DDSD_PIXELFORMAT: Final = 0x1000
DDSD_MIPMAPCOUNT: Final = 0x20000
DDSD_LINEARSIZE: Final = 0x80000
DDSD_DEPTH: Final = 0x800000

DDSCAPS_COMPLEX: Final = 0x8
DDSCAPS_TEXTURE: Final = 0x1000
DDSCAPS_MIPMAP: Final = 0x400000

DDSCAPS2_CUBEMAP: Final = 0x200
DDSCAPS2_CUBEMAP_POSITIVEX: Final = 0x400
DDSCAPS2_CUBEMAP_NEGATIVEX: Final = 0x800
DDSCAPS2_CUBEMAP_POSITIVEY: Final = 0x1000
DDSCAPS2_CUBEMAP_NEGATIVEY: Final = 0x2000
DDSCAPS2_CUBEMAP_POSITIVEZ: Final = 0x4000
DDSCAPS2_CUBEMAP_NEGATIVEZ: Final = 0x8000
DDSCAPS2_VOLUME: Final = 0x200000

DDPF_ALPHAPIXELS: Final = 0x1
DDPF_ALPHA: Final = 0x2
DDPF_FOURCC: Final = 0x4
DDPF_PALETTEINDEXED8: Final = 0x20
DDPF_RGB: Final = 0x40
DDPF_LUMINANCE: Final = 0x20000

DDS_FOURCC: Final = 0x4
DDS_RGB: Final = 0x40
DDS_RGBA: Final = 0x41
DDS_LUMINANCE: Final = 0x20000
DDS_LUMINANCEA: Final = 0x20001
DDS_ALPHA: Final = 0x2
DDS_PAL8: Final = 0x20

DDS_HEADER_FLAGS_TEXTURE: int
DDS_HEADER_FLAGS_MIPMAP: int
DDS_HEADER_FLAGS_VOLUME: int
DDS_HEADER_FLAGS_PITCH: int
DDS_HEADER_FLAGS_LINEARSIZE: int
DDS_HEIGHT: int
DDS_WIDTH: int
DDS_SURFACE_FLAGS_TEXTURE: int
DDS_SURFACE_FLAGS_MIPMAP: int
DDS_SURFACE_FLAGS_CUBEMAP: int
DDS_CUBEMAP_POSITIVEX: int
DDS_CUBEMAP_NEGATIVEX: int
DDS_CUBEMAP_POSITIVEY: int
DDS_CUBEMAP_NEGATIVEY: int
DDS_CUBEMAP_POSITIVEZ: int
DDS_CUBEMAP_NEGATIVEZ: int

DXT1_FOURCC: Final = 0x31545844
DXT3_FOURCC: Final = 0x33545844
DXT5_FOURCC: Final = 0x35545844

DXGI_FORMAT_R8G8B8A8_TYPELESS: Final = 27
DXGI_FORMAT_R8G8B8A8_UNORM: Final = 28
DXGI_FORMAT_R8G8B8A8_UNORM_SRGB: Final = 29
DXGI_FORMAT_BC5_TYPELESS: Final = 82
DXGI_FORMAT_BC5_UNORM: Final = 83
DXGI_FORMAT_BC5_SNORM: Final = 84
DXGI_FORMAT_BC6H_UF16: Final = 95
DXGI_FORMAT_BC6H_SF16: Final = 96
DXGI_FORMAT_BC7_TYPELESS: Final = 97
DXGI_FORMAT_BC7_UNORM: Final = 98
DXGI_FORMAT_BC7_UNORM_SRGB: Final = 99

class DdsImageFile(ImageFile):
    format: ClassVar[Literal["DDS"]]
    format_description: ClassVar[str]
    def load_seek(self, pos) -> None: ...



from _typeshed import Incomplete
from typing import ClassVar
from typing_extensions import Literal

from .ImageFile import ImageFile

xpm_head: Incomplete

class XpmImageFile(ImageFile):
    format: ClassVar[Literal["XPM"]]
    format_description: ClassVar[str]
    def load_read(self, bytes): ...



from _typeshed import Incomplete
from typing import ClassVar
from typing_extensions import Literal

from .ImageFile import ImageFile

COMMENT: str
DATE: str
EQUIPMENT: str
FRAMES: str
LUT: str
NAME: str
SCALE: str
SIZE: str
MODE: str
TAGS: Incomplete
OPEN: Incomplete
split: Incomplete

def number(s): ...

class ImImageFile(ImageFile):
    format: ClassVar[Literal["IM"]]
    format_description: ClassVar[str]
    @property
    def n_frames(self): ...
    @property
    def is_animated(self): ...
    frame: Incomplete
    fp: Incomplete
    tile: Incomplete
    def seek(self, frame) -> None: ...
    def tell(self): ...

SAVE: Incomplete



from _typeshed import FileDescriptorOrPath, Incomplete, SupportsRead
from enum import IntEnum
from typing import Protocol
from typing_extensions import Final, Literal

from PIL.Image import Transpose

class Layout(IntEnum):
    BASIC: Literal[0]
    RAQM: Literal[1]

MAX_STRING_LENGTH: Final[int] = 1_000_000

class _Font(Protocol):
    def getmask(self, text: str | bytes, mode: str = ..., direction=..., features=...): ...

class ImageFont:
    def getmask(self, text: str | bytes, mode: str = "", direction=..., features=...): ...
    def getbbox(self, text, *args, **kwargs): ...
    def getlength(self, text, *args, **kwargs): ...

class FreeTypeFont:
    path: str | bytes | SupportsRead[bytes] | None
    size: int
    index: int
    encoding: str
    layout_engine: Layout
    font_bytes: bytes  # Only exists under some circumstances.
    font: Incomplete
    def __init__(
        self,
        font: str | bytes | SupportsRead[bytes] | None = None,
        size: int = 10,
        index: int = 0,
        encoding: str = "",
        layout_engine: Layout | None = None,
    ) -> None: ...
    def getname(self) -> tuple[str, str]: ...
    def getmetrics(self) -> tuple[int, int]: ...
    def getlength(
        self,
        text: str | bytes,
        mode: str = "",
        direction: Literal["ltr", "rtl", "ttb"] | None = None,
        features: Incomplete | None = None,
        language: str | None = None,
    ) -> float: ...
    def getbbox(
        self,
        text: str | bytes,
        mode: str = "",
        direction=None,
        features=None,
        language: str | None = None,
        stroke_width: int = 0,
        anchor: str | None = None,
    ) -> tuple[int, int, int, int]: ...
    def getmask(
        self,
        text: str | bytes,
        mode: str = "",
        direction: Literal["ltr", "rtl", "ttb"] | None = None,
        features: Incomplete | None = None,
        language: str | None = None,
        stroke_width: float = 0,
        anchor: str | None = None,
        ink=0,
        start: tuple[float, float] | None = None,
    ): ...
    def getmask2(
        self,
        text: str | bytes,
        mode: str = "",
        direction: Literal["ltr", "rtl", "ttb"] | None = None,
        features: Incomplete | None = None,
        language: str | None = None,
        stroke_width: float = 0,
        anchor: str | None = None,
        ink=0,
        start: tuple[float, float] | None = None,
        *args,
        **kwargs,
    ): ...
    def font_variant(
        self,
        font: str | bytes | SupportsRead[bytes] | None = None,
        size: int | None = None,
        index: int | None = None,
        encoding: str | None = None,
        layout_engine: Layout | None = None,
    ) -> FreeTypeFont: ...
    def get_variation_names(self): ...
    def set_variation_by_name(self, name): ...
    def get_variation_axes(self): ...
    def set_variation_by_axes(self, axes): ...

class TransposedFont:
    font: _Font
    orientation: Transpose | None
    def __init__(self, font: _Font, orientation: Transpose | None = None) -> None: ...
    def getmask(self, text: str | bytes, mode: str = "", *args, **kwargs): ...
    def getbbox(self, text, *args, **kwargs): ...
    def getlength(self, text, *args, **kwargs): ...

def load(filename: FileDescriptorOrPath) -> ImageFont: ...
def truetype(
    font: str | bytes | SupportsRead[bytes] | None = None,
    size: int = 10,
    index: int = 0,
    encoding: str = "",
    layout_engine: Layout | None = None,
) -> FreeTypeFont: ...
def load_path(filename: str | bytes) -> ImageFont: ...
def load_default() -> ImageFont: ...



from .Image import Image

class _Enhance:
    def enhance(self, factor: float) -> Image: ...

class Color(_Enhance):
    image: Image
    intermediate_mode: str
    degenerate: Image
    def __init__(self, image: Image) -> None: ...

class Contrast(_Enhance):
    image: Image
    degenerate: Image
    def __init__(self, image: Image) -> None: ...

class Brightness(_Enhance):
    image: Image
    degenerate: Image
    def __init__(self, image: Image) -> None: ...

class Sharpness(_Enhance):
    image: Image
    degenerate: Image
    def __init__(self, image: Image) -> None: ...



from _typeshed import Incomplete

from .FontFile import FontFile

bdf_slant: Incomplete
bdf_spacing: Incomplete

def bdf_char(f): ...

class BdfFontFile(FontFile):
    def __init__(self, fp) -> None: ...



from _typeshed import Incomplete
from collections.abc import Container, Sequence
from typing import Any, overload
from typing_extensions import Literal, TypeAlias

from .Image import Image
from .ImageColor import _Ink
from .ImageFont import _Font

_XY: TypeAlias = Sequence[float | tuple[float, float]]
_Outline: TypeAlias = Any

class ImageDraw:
    font: Incomplete
    palette: Incomplete
    im: Incomplete
    draw: Incomplete
    mode: Incomplete
    ink: Incomplete
    fontmode: str
    fill: bool
    def __init__(self, im: Image, mode: str | None = None) -> None: ...
    def getfont(self): ...
    def arc(self, xy: _XY, start: float, end: float, fill: _Ink | None = None, width: float = 1) -> None: ...
    def bitmap(self, xy: _XY, bitmap: Image, fill: _Ink | None = None) -> None: ...
    def chord(
        self, xy: _XY, start: float, end: float, fill: _Ink | None = None, outline: _Ink | None = None, width: float = 1
    ) -> None: ...
    def ellipse(self, xy: _XY, fill: _Ink | None = None, outline: _Ink | None = None, width: float = 1) -> None: ...
    def line(self, xy: _XY, fill: _Ink | None = None, width: float = 0, joint: Literal["curve"] | None = None) -> None: ...
    def shape(self, shape: _Outline, fill: _Ink | None = None, outline: _Ink | None = None) -> None: ...
    def pieslice(
        self,
        xy: tuple[tuple[float, float], tuple[float, float]],
        start: float,
        end: float,
        fill: _Ink | None = None,
        outline: _Ink | None = None,
        width: float = 1,
    ) -> None: ...
    def point(self, xy: _XY, fill: _Ink | None = None) -> None: ...
    def polygon(self, xy: _XY, fill: _Ink | None = None, outline: _Ink | None = None, width: float = 1) -> None: ...
    def regular_polygon(
        self,
        bounding_circle: tuple[float, float] | tuple[float, float, float] | list[int],
        n_sides: int,
        rotation: float = 0,
        fill: _Ink | None = None,
        outline: _Ink | None = None,
        width: float = 1,
    ) -> None: ...
    def rectangle(
        self,
        xy: tuple[float, float, float, float] | tuple[tuple[float, float], tuple[float, float]],
        fill: _Ink | None = None,
        outline: _Ink | None = None,
        width: float = 1,
    ) -> None: ...
    def rounded_rectangle(
        self,
        xy: tuple[float, float, float, float] | tuple[tuple[float, float], tuple[float, float]],
        radius: float = 0,
        fill: _Ink | None = None,
        outline: _Ink | None = None,
        width: float = 1,
        *,
        corners: tuple[bool, bool, bool, bool] | None = None,
    ) -> None: ...
    def text(
        self,
        xy: tuple[float, float],
        text: str | bytes,
        fill: _Ink | None = None,
        font: _Font | None = None,
        anchor: str | None = None,
        spacing: float = 4,
        align: Literal["left", "center", "right"] = "left",
        direction: Literal["rtl", "ltr", "ttb"] | None = None,
        features: Sequence[str] | None = None,
        language: str | None = None,
        stroke_width: int = 0,
        stroke_fill: _Ink | None = None,
        embedded_color: bool = False,
        *args,
        **kwargs,
    ) -> None: ...
    def multiline_text(
        self,
        xy: tuple[float, float],
        text: str | bytes,
        fill: _Ink | None = None,
        font: _Font | None = None,
        anchor: str | None = None,
        spacing: float = 4,
        align: Literal["left", "center", "right"] = "left",
        direction: Literal["rtl", "ltr", "ttb"] | None = None,
        features: Incomplete | None = None,
        language: str | None = None,
        stroke_width: int = 0,
        stroke_fill: _Ink | None = None,
        embedded_color: bool = False,
    ) -> None: ...
    def textlength(
        self,
        text: str | bytes,
        font: _Font | None = None,
        direction: Literal["rtl", "ltr", "ttb"] | None = None,
        features: Sequence[str] | None = None,
        language: str | None = None,
        embedded_color: bool = False,
    ) -> float: ...
    def textbbox(
        self,
        xy: tuple[float, float],
        text: str | bytes,
        font: _Font | None = None,
        anchor: str | None = None,
        spacing: float = 4,
        align: Literal["left", "center", "right"] = "left",
        direction: Literal["rtl", "ltr", "ttb"] | None = None,
        features: Incomplete | None = None,
        language: str | None = None,
        stroke_width: int = 0,
        embedded_color: bool = False,
    ) -> tuple[int, int, int, int]: ...
    def multiline_textbbox(
        self,
        xy: tuple[float, float],
        text: str | bytes,
        font: _Font | None = None,
        anchor: str | None = None,
        spacing: float = 4,
        align: Literal["left", "center", "right"] = "left",
        direction: Literal["rtl", "ltr", "ttb"] | None = None,
        features: Incomplete | None = None,
        language: str | None = None,
        stroke_width: int = 0,
        embedded_color: bool = False,
    ) -> tuple[int, int, int, int]: ...

def Draw(im: Image, mode: str | None = None) -> ImageDraw: ...
def Outline() -> _Outline: ...
@overload
def getdraw(im: None = None, hints: Container[Literal["nicest"]] | None = None) -> tuple[None, Any]: ...
@overload
def getdraw(im: Image, hints: Container[Literal["nicest"]] | None = None) -> tuple[Image, Any]: ...
def floodfill(image: Image, xy: tuple[float, float], value, border=None, thresh: float = 0) -> None: ...



from _typeshed import Incomplete

from .Image import Image
from .ImageDraw import ImageDraw

class Pen:
    color: Incomplete
    width: Incomplete
    def __init__(self, color, width: int = 1, opacity: int = 255) -> None: ...

class Brush:
    color: Incomplete
    def __init__(self, color, opacity: int = 255) -> None: ...

class Font:
    color: Incomplete
    font: Incomplete
    def __init__(self, color, file, size: int = 12) -> None: ...

class Draw:
    draw: ImageDraw
    image: Image
    transform: Incomplete | None
    def __init__(self, image, size: Incomplete | None = None, color: Incomplete | None = None) -> None: ...
    def flush(self) -> Image: ...
    def render(self, op, xy, pen, brush: Incomplete | None = None) -> None: ...
    def settransform(self, offset) -> None: ...
    def arc(self, xy, start, end, *options) -> None: ...
    def chord(self, xy, start, end, *options) -> None: ...
    def ellipse(self, xy, *options) -> None: ...
    def line(self, xy, *options) -> None: ...
    def pieslice(self, xy, start, end, *options) -> None: ...
    def polygon(self, xy, *options) -> None: ...
    def rectangle(self, xy, *options) -> None: ...
    def text(self, xy, text, font) -> None: ...
    def textbbox(self, xy, text, font): ...
    def textlength(self, text, font): ...



import sys
from _typeshed import Incomplete
from collections.abc import Callable, MutableMapping
from fractions import Fraction, _ComparableNum
from numbers import Rational
from types import TracebackType
from typing import ClassVar, overload
from typing_extensions import Final, Literal, SupportsIndex, TypeAlias

from ._imaging import _PixelAccessor
from .ImageFile import ImageFile

_OpenInfo: TypeAlias = tuple[Literal[b"II", b"MM"], int, tuple[int, ...], int, tuple[int, ...], tuple[int, ...]]

# These are meant to be overridable
READ_LIBTIFF: bool
WRITE_LIBTIFF: bool
IFD_LEGACY_API: bool
STRIP_SIZE: int

II: Final = b"II"
MM: Final = b"MM"
IMAGEWIDTH: Final = 256
IMAGELENGTH: Final = 257
BITSPERSAMPLE: Final = 258
COMPRESSION: Final = 259
PHOTOMETRIC_INTERPRETATION: Final = 262
FILLORDER: Final = 266
IMAGEDESCRIPTION: Final = 270
STRIPOFFSETS: Final = 273
SAMPLESPERPIXEL: Final = 277
ROWSPERSTRIP: Final = 278
STRIPBYTECOUNTS: Final = 279
X_RESOLUTION: Final = 282
Y_RESOLUTION: Final = 283
PLANAR_CONFIGURATION: Final = 284
RESOLUTION_UNIT: Final = 296
TRANSFERFUNCTION: Final = 301
SOFTWARE: Final = 305
DATE_TIME: Final = 306
ARTIST: Final = 315
PREDICTOR: Final = 317
COLORMAP: Final = 320
TILEWIDTH: Final = 322
TILELENGTH: Final = 323
TILEOFFSETS: Final = 324
TILEBYTECOUNTS: Final = 325
SUBIFD: Final = 330
EXTRASAMPLES: Final = 338
SAMPLEFORMAT: Final = 339
JPEGTABLES: Final = 347
YCBCRSUBSAMPLING: Final = 530
REFERENCEBLACKWHITE: Final = 532
COPYRIGHT: Final = 33432
IPTC_NAA_CHUNK: Final = 33723
PHOTOSHOP_CHUNK: Final = 34377
ICCPROFILE: Final = 34675
EXIFIFD: Final = 34665
XMP: Final = 700
JPEGQUALITY: Final = 65537
IMAGEJ_META_DATA_BYTE_COUNTS: Final = 50838
IMAGEJ_META_DATA: Final = 50839
COMPRESSION_INFO: Final[dict[int, str]]
COMPRESSION_INFO_REV: Final[dict[str, int]]
OPEN_INFO: Final[dict[_OpenInfo, tuple[str, str]]]
MAX_SAMPLESPERPIXEL: Final = 6
PREFIXES: Final[list[bytes]]

class IFDRational(Rational):
    def __init__(self, value, denominator: int = 1) -> None: ...
    @property
    def numerator(a): ...
    @property
    def denominator(a): ...
    def limit_rational(self, max_denominator): ...
    def __hash__(self) -> int: ...
    def __eq__(self, other): ...
    # The following methods are aliased from fractions.Fraction:
    @overload
    def __add__(a, b: int | Fraction) -> Fraction: ...
    @overload
    def __add__(a, b: float) -> float: ...
    @overload
    def __add__(a, b: complex) -> complex: ...
    @overload
    def __radd__(b, a: int | Fraction) -> Fraction: ...
    @overload
    def __radd__(b, a: float) -> float: ...
    @overload
    def __radd__(b, a: complex) -> complex: ...
    @overload
    def __sub__(a, b: int | Fraction) -> Fraction: ...
    @overload
    def __sub__(a, b: float) -> float: ...
    @overload
    def __sub__(a, b: complex) -> complex: ...
    @overload
    def __rsub__(b, a: int | Fraction) -> Fraction: ...
    @overload
    def __rsub__(b, a: float) -> float: ...
    @overload
    def __rsub__(b, a: complex) -> complex: ...
    @overload
    def __mul__(a, b: int | Fraction) -> Fraction: ...
    @overload
    def __mul__(a, b: float) -> float: ...
    @overload
    def __mul__(a, b: complex) -> complex: ...
    @overload
    def __rmul__(b, a: int | Fraction) -> Fraction: ...
    @overload
    def __rmul__(b, a: float) -> float: ...
    @overload
    def __rmul__(b, a: complex) -> complex: ...
    @overload
    def __truediv__(a, b: int | Fraction) -> Fraction: ...
    @overload
    def __truediv__(a, b: float) -> float: ...
    @overload
    def __truediv__(a, b: complex) -> complex: ...
    @overload
    def __rtruediv__(b, a: int | Fraction) -> Fraction: ...
    @overload
    def __rtruediv__(b, a: float) -> float: ...
    @overload
    def __rtruediv__(b, a: complex) -> complex: ...
    @overload
    def __floordiv__(a, b: int | Fraction) -> int: ...
    @overload
    def __floordiv__(a, b: float) -> float: ...
    @overload
    def __rfloordiv__(b, a: int | Fraction) -> int: ...
    @overload
    def __rfloordiv__(b, a: float) -> float: ...
    @overload
    def __mod__(a, b: int | Fraction) -> Fraction: ...
    @overload
    def __mod__(a, b: float) -> float: ...
    @overload
    def __rmod__(b, a: int | Fraction) -> Fraction: ...
    @overload
    def __rmod__(b, a: float) -> float: ...
    @overload
    def __pow__(a, b: int) -> Fraction: ...
    @overload
    def __pow__(a, b: float | Fraction) -> float: ...
    @overload
    def __pow__(a, b: complex) -> complex: ...
    @overload
    def __rpow__(b, a: float | Fraction) -> float: ...
    @overload
    def __rpow__(b, a: complex) -> complex: ...
    def __pos__(a) -> Fraction: ...
    def __neg__(a) -> Fraction: ...
    def __abs__(a) -> Fraction: ...
    def __trunc__(a) -> int: ...
    def __lt__(a, b: _ComparableNum) -> bool: ...
    def __gt__(a, b: _ComparableNum) -> bool: ...
    def __le__(a, b: _ComparableNum) -> bool: ...
    def __ge__(a, b: _ComparableNum) -> bool: ...
    def __bool__(a) -> bool: ...
    def __ceil__(a) -> int: ...
    def __floor__(a) -> int: ...
    @overload
    def __round__(self, ndigits: None = None) -> int: ...
    @overload
    def __round__(self, ndigits: int) -> Fraction: ...
    if sys.version_info >= (3, 11):
        def __int__(a, _index: Callable[[SupportsIndex], int] = ...) -> int: ...

class ImageFileDirectory_v2(MutableMapping[int, Incomplete]):
    group: int | None
    tagtype: dict[int, int]
    def __init__(
        self, ifh: bytes = b"II*\x00\x00\x00\x00\x00", prefix: bytes | None = None, group: int | None = None
    ) -> None: ...
    @property
    def prefix(self) -> bytes: ...
    @property
    def offset(self) -> int | None: ...
    @property
    def legacy_api(self) -> bool: ...
    def reset(self) -> None: ...
    def named(self): ...
    def __len__(self) -> int: ...
    def __getitem__(self, tag): ...
    def __contains__(self, tag): ...
    def __setitem__(self, tag, value) -> None: ...
    def __delitem__(self, tag) -> None: ...
    def __iter__(self): ...
    def load_byte(self, data, legacy_api: bool = True): ...
    def write_byte(self, data): ...
    def load_string(self, data, legacy_api: bool = True): ...
    def write_string(self, value: int | str | bytes) -> bytes: ...
    def load_rational(self, data, legacy_api: bool = True): ...
    def write_rational(self, *values): ...
    def load_undefined(self, data, legacy_api: bool = True): ...
    def write_undefined(self, value): ...
    def load_signed_rational(self, data, legacy_api: bool = True): ...
    def write_signed_rational(self, *values): ...
    def load(self, fp) -> None: ...
    def tobytes(self, offset: int = 0): ...
    def save(self, fp): ...
    load_double: Incomplete
    load_float: Incomplete
    load_long: Incomplete
    load_long8: Incomplete
    load_short: Incomplete
    load_signed_byte: Incomplete
    load_signed_long: Incomplete
    load_signed_short: Incomplete
    write_double: Incomplete
    write_float: Incomplete
    write_long: Incomplete
    write_long8: Incomplete
    write_short: Incomplete
    write_signed_byte: Incomplete
    write_signed_long: Incomplete
    write_signed_short: Incomplete

class ImageFileDirectory_v1(ImageFileDirectory_v2):
    def __init__(self, *args, **kwargs) -> None: ...
    @property
    def tags(self): ...
    @property
    def tagdata(self): ...
    tagtype: dict[int, int]
    @classmethod
    def from_v2(cls, original): ...
    def to_v2(self): ...
    def __contains__(self, tag): ...
    def __len__(self) -> int: ...
    def __iter__(self): ...
    def __setitem__(self, tag, value) -> None: ...
    def __getitem__(self, tag): ...

ImageFileDirectory = ImageFileDirectory_v1

class TiffImageFile(ImageFile):
    format: ClassVar[Literal["TIFF", "MIC"]]
    format_description: ClassVar[str]
    tag_v2: Incomplete
    tag: Incomplete
    def __init__(self, fp: Incomplete | None = None, filename: Incomplete | None = None) -> None: ...
    @property
    def n_frames(self): ...
    im: Incomplete
    def seek(self, frame) -> None: ...
    def tell(self): ...
    def getxmp(self): ...
    def get_photoshop_blocks(self): ...
    def load(self) -> _PixelAccessor: ...
    def load_end(self) -> None: ...

SAVE_INFO: Incomplete

class AppendingTiffWriter:
    fieldSizes: Incomplete
    Tags: Incomplete
    f: Incomplete
    close_fp: bool
    name: Incomplete
    beginning: Incomplete
    def __init__(self, fn, new: bool = False) -> None: ...
    whereToWriteNewIFDOffset: Incomplete
    offsetOfNewPage: int
    IIMM: Incomplete
    isFirst: bool
    def setup(self) -> None: ...
    def finalize(self) -> None: ...
    def newFrame(self) -> None: ...
    def __enter__(self): ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_value: BaseException | None, traceback: TracebackType | None
    ) -> Literal[False]: ...
    def tell(self): ...
    def seek(self, offset, whence=0): ...
    def goToEnd(self) -> None: ...
    endian: Incomplete
    longFmt: Incomplete
    shortFmt: Incomplete
    tagFormat: Incomplete
    def setEndian(self, endian) -> None: ...
    def skipIFDs(self) -> None: ...
    def write(self, data): ...
    def readShort(self): ...
    def readLong(self): ...
    def rewriteLastShortToLong(self, value) -> None: ...
    def rewriteLastShort(self, value) -> None: ...
    def rewriteLastLong(self, value) -> None: ...
    def writeShort(self, value) -> None: ...
    def writeLong(self, value) -> None: ...
    def close(self) -> None: ...
    def fixIFD(self) -> None: ...
    def fixOffsets(self, count, isShort: bool = False, isLong: bool = False) -> None: ...



from _typeshed import Incomplete
from typing import ClassVar
from typing_extensions import Literal

from .ImageFile import ImageFile

def isInt(f: object) -> Literal[0, 1]: ...

iforms: Incomplete

def isSpiderHeader(t): ...
def isSpiderImage(filename): ...

class SpiderImageFile(ImageFile):
    format: ClassVar[Literal["SPIDER"]]
    format_description: ClassVar[str]
    @property
    def n_frames(self): ...
    @property
    def is_animated(self): ...
    def tell(self): ...
    stkoffset: Incomplete
    fp: Incomplete
    def seek(self, frame) -> None: ...
    def convert2byte(self, depth: int = 255): ...
    def tkPhotoImage(self): ...

def loadImageSeries(filelist: Incomplete | None = None): ...
def makeSpiderHeader(im): ...



from _typeshed import Incomplete
from typing import ClassVar
from typing_extensions import Literal

from .ImageFile import ImageFile

class PcdImageFile(ImageFile):
    format: ClassVar[Literal["PCD"]]
    format_description: ClassVar[str]
    im: Incomplete
    def load_end(self) -> None: ...



from _typeshed import Incomplete
from typing import ClassVar
from typing_extensions import Literal

from .PcxImagePlugin import PcxImageFile

MAGIC: int

class DcxImageFile(PcxImageFile):
    format: ClassVar[Literal["DCX"]]
    frame: Incomplete
    fp: Incomplete
    def seek(self, frame) -> None: ...
    def tell(self): ...
